/*
 * Copyright 2002-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jboss.seam.mock;

import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.io.OutputStream;


/**
 * Delegating implementation of {@link javax.servlet.ServletOutputStream}.
 * <p/>
 * <p>Used by {@link MockHttpServletResponse}; typically not directly
 * used for testing application controllers.
 *
 * @author Juergen Hoeller
 * @see MockHttpServletResponse
 * @since 1.0.2
 */
public class DelegatingServletOutputStream extends ServletOutputStream
{

   private final OutputStream targetStream;


   /**
    * Create a DelegatingServletOutputStream for the given target stream.
    *
    * @param targetStream the target stream (never <code>null</code>)
    */
   public DelegatingServletOutputStream(OutputStream targetStream)
   {
      this.targetStream = targetStream;
   }

   /**
    * Return the underlying target stream (never <code>null</code>).
    */
   public final OutputStream getTargetStream()
   {
      return this.targetStream;
   }


   public void write(int b) throws IOException
   {
      this.targetStream.write(b);
   }

   public void flush() throws IOException
   {
      super.flush();
      this.targetStream.flush();
   }

   public void close() throws IOException
   {
      super.close();
      this.targetStream.close();
   }

}